@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See the COPYING
@@ -105,7 +105,7 @@ static void sigcont_handler(int sig)
 }
 #endif
 
-static void show_info(int what, struct xmp_module_info *mi)
+static void show_info(int what, struct xmp_module_info *mi, int mode)
 {
 	report("\r%78.78s\n", " ");
 	switch (what) {
@@ -125,7 +125,7 @@ static void show_info(int what, struct xmp_module_info *mi)
 		info_comment(mi);
 		break;
 	case 'm':
-		info_mod(mi);
+		info_mod(mi, mode);
 		break;
 	}
 }
@@ -155,8 +155,9 @@ static void check_pause(xmp_context xc, struct control *ctl,
 			usleep(100000);
 			read_command(xc, mi, ctl);
 			if (ctl->display) {
-				show_info(ctl->display, mi);
 				if (verbose) {
+					int mode = xmp_get_player(xc, XMP_PLAYER_MODE);
+					show_info(ctl->display, mi, mode);
 					info_frame(mi, fi, ctl, 1);
 				}
 				ctl->display = 0;
@@ -166,6 +167,41 @@ static void check_pause(xmp_context xc, struct control *ctl,
 	}
 }
 
+static void get_mixer_type(int t, struct options *opt, char *buf, size_t size)
+{
+	char *fmt;
+
+	fmt = opt->format & XMP_FORMAT_MONO ? "mono" : "stereo";
+
+	switch (t) {
+	case XMP_MIXER_STANDARD: {
+		char *itp;
+		switch (opt->interp) {
+		case XMP_INTERP_NEAREST:
+			itp = "non-interpolated";
+			break;
+		case XMP_INTERP_LINEAR:
+			itp = "linear interpolated";
+			break;
+		case XMP_INTERP_SPLINE:
+			itp = "cubic spline interpolated";
+			break;
+		default:
+			itp = "unknown interpolation";
+		}
+		snprintf(buf, size, "%s %s mixer", fmt, itp);
+		break; }
+	case XMP_MIXER_A500:
+		snprintf(buf, size, "Amiga 500 %s mixer", fmt);
+		break;
+	case XMP_MIXER_A500F:
+		snprintf(buf, size, "Amiga 500 %s mixer (led filter: on)", fmt);
+		break;
+	default:
+		snprintf(buf, size, "unknown %s mixer", fmt);
+	}
+}
+
 static void load_error(char *name, char *filename, int val)
 {
 	char *msg;
@@ -226,9 +262,12 @@ int main(int argc, char **argv)
 	opt.rate = 44100;
 	opt.mix = -1;
 	opt.defpan = 50;
+	opt.numvoices = 128;
 	opt.driver_id = NULL;
 	opt.interp = XMP_INTERP_SPLINE;
 	opt.dsp = XMP_DSP_LOWPASS;
+	opt.player_mode = XMP_MODE_AUTO;
+	opt.amplify = 1;
 
 	/* read configuration file */
 	if (!opt.norc) {
@@ -251,6 +290,13 @@ int main(int argc, char **argv)
 		exit(EXIT_FAILURE);
 	}
 
+	if (opt.player_mode < 0) {
+		fprintf(stderr, "%s: unknown player mode\n"
+			"Use `%s --help' for more information.\n",
+			argv[0], argv[0]);
+		exit(EXIT_FAILURE);
+	}
+
 	if (opt.silent) {
 		opt.driver_id = "null";
 	}
@@ -272,7 +318,7 @@ int main(int argc, char **argv)
 
 	if (opt.verbose > 0) {
 		report("Extended Module Player " VERSION "\n"
-			"Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr\n");
+			"Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr\n");
 
 		report("Using %s\n", sound->description);
 
@@ -322,6 +368,7 @@ int main(int argc, char **argv)
 
 	control.loop = opt.loop;
 	control.explore = opt.explore;
+	control.amiga = opt.amiga;
 	first = optind;
 
     play_all:
@@ -340,9 +387,8 @@ int main(int argc, char **argv)
 				argv[optind], optind - first + 1, argc - first);
 		}
 
-#if XMP_VERCODE >= 0x040300
+		/* these must be set before loading the module */
 		xmp_set_player(xc, XMP_PLAYER_DEFPAN, opt.defpan);
-#endif
 
 		/* load module */
 
@@ -366,6 +412,10 @@ int main(int argc, char **argv)
 			read_modconf(&opt, mi.md5);
 		}
 
+
+		/* these must be set before xmp_start_player() */
+		xmp_set_player(xc, XMP_PLAYER_VOICES, opt.numvoices);
+
 		/* check sequence */
 
 		if (opt.sequence) {
@@ -395,6 +445,10 @@ int main(int argc, char **argv)
 		if (xmp_start_player(xc, opt.rate, opt.format) == 0) {
 			xmp_set_player(xc, XMP_PLAYER_INTERP, opt.interp);
 			xmp_set_player(xc, XMP_PLAYER_DSP, opt.dsp);
+			if (opt.player_mode != XMP_MODE_AUTO) {
+				xmp_set_player(xc, XMP_PLAYER_MODE,
+							opt.player_mode);
+			}
 
 			played = 1;
 
@@ -429,12 +483,14 @@ int main(int argc, char **argv)
 			set_flag(flags, opt.vblank, XMP_FLAGS_VBLANK);
 			set_flag(flags, opt.fx9bug, XMP_FLAGS_FX9BUG);
 			set_flag(flags, opt.fixloop, XMP_FLAGS_FIXLOOP);
+			set_flag(flags, control.amiga, XMP_FLAGS_A500);
 			xmp_set_player(xc, XMP_PLAYER_CFLAGS, flags);
 
 			/* Show module data */
 
 			if (opt.verbose > 0) {
-				info_mod(&mi);
+				int mode = xmp_get_player(xc, XMP_PLAYER_MODE);
+				info_mod(&mi, mode);
 			}
 
 			if (opt.verbose > 1) {
@@ -458,6 +514,8 @@ int main(int argc, char **argv)
 				int old_loop = fi.loop_count;
 				
 				xmp_get_frame_info(xc, &fi);
+				control.mixer_type = xmp_get_player(
+						xc, XMP_PLAYER_MIXER_TYPE);
 
 				/* Check loop */
 
@@ -487,14 +545,26 @@ int main(int argc, char **argv)
 					read_command(xc, &mi, &control);
 
 					if (control.display) {
-						show_info(control.display, &mi);
+						int mode = xmp_get_player(xc,
+							XMP_PLAYER_MODE);
+						show_info(control.display, &mi,
+									mode);
 						control.display = 0;
 						refresh_status = 1;
 					}
 
-					if (control.cur_seq) {
+					
+					switch (control.cur_info) {
+					case 'X': {
+						char buf[80];
+ 						get_mixer_type(control.mixer_type, &opt, buf, 80);
+						info_message("Mixer type: %s", buf);
+						control.cur_info = 0;
+						break; }
+					case 'Z':
 						info_message("Current sequence: %d (start at position %02X)", control.sequence, mi.seq_data[control.sequence].entry_point);
-						control.cur_seq = 0;
+						control.cur_info = 0;
+						break;
 					}
 				}
 
