@@ -1,5 +1,5 @@
 /* Extended Module Player
- * Copyright (C) 1996-2014 Claudio Matsuoka and Hipolito Carraro Jr
+ * Copyright (C) 1996-2016 Claudio Matsuoka and Hipolito Carraro Jr
  *
  * This file is part of the Extended Module Player and is distributed
  * under the terms of the GNU General Public License. See the COPYING
@@ -115,12 +115,14 @@ int read_config(struct options *o)
 		getval_yn("reverse", o->reverse, 1);
 		getval_no("amplify", o->amplify);
 		getval_no("mix", o->mix);
+		getval_no("mixer_voices", o->numvoices);
 		getval_no("default_pan", o->defpan);
 		/*getval_no("chorus", o->chorus);
 		getval_no("reverb", o->reverb);*/
 		getval_no("srate", o->rate);
 		/*getval_no("time", o->time);
 		getval_no("verbosity", o->verbosity);*/
+		getval_yn("amiga", o->amiga, 1);
 
 		if (!strcmp(var, "driver")) {
 			strncpy(driver, val, 31);
@@ -190,7 +192,9 @@ static void parse_modconf(struct options *o, char *confname, unsigned char *md5)
 {
 	FILE *rc;
 	char *hash, *var, *val, line[256];
+	struct player_mode *pm;
 	int active = 0;
+	int mono = 0;
 
 	if ((rc = fopen(confname, "r")) == NULL)
 		return;
@@ -226,17 +230,18 @@ static void parse_modconf(struct options *o, char *confname, unsigned char *md5)
 
 		val = strtok(NULL, " \t\n");
 
-		getval_yn("8bit", o->format, XMP_FORMAT_8BIT);
-		getval_yn("mono", o->format, XMP_FORMAT_MONO);
 		getval_yn("filter", o->dsp, XMP_DSP_LOWPASS);
-		getval_yn("loop", o->loop, XMP_DSP_LOWPASS);
+		getval_yn("loop", o->loop, 1);
+		getval_yn("mono", mono, 1);
 		getval_yn("reverse", o->reverse, 1);
 		getval_no("amplify", o->amplify);
 		getval_no("mix", o->mix);
+		getval_no("mixer_voices", o->numvoices);
 
 		getval_tristate("fixloop", o->fixloop);
 		getval_tristate("fx9bug", o->fx9bug);
 		getval_tristate("vblank", o->vblank);
+		getval_tristate("amiga", o->amiga);
 
 		if (!strcmp(var, "interpolation")) {
 			if (!strcmp(val, "nearest")) {
@@ -250,6 +255,23 @@ static void parse_modconf(struct options *o, char *confname, unsigned char *md5)
 					"type \"%s\"\n", confname, val);
 			}
 		}
+
+		if (!strcmp(var, "mode")) {
+			for (pm = pmode; pm->name != NULL; pm++) {
+				if (!strcmp(val, pm->name)) {
+					o->player_mode = pm->mode;
+					break;
+				}
+				fprintf(stderr, "%s: unknown player mode "
+					"\"%s\"\n", confname, val);
+			}
+		}
+
+		fprintf(stderr, "%s: unknown option \"%s\"\n", confname, var);
+	}
+
+	if (mono) {
+		o->mix = 0;
 	}
 
 	fclose(rc);
