@@ -1,7 +1,3 @@
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #ifndef HAVE_USLEEP
 
 #ifdef HAVE_SELECT
